@@ -139,6 +139,11 @@ def test_glob_return_subfolders(server):
 
 
 def test_isdir(server):
+    h = fsspec.filesystem("http", headers={"give_mimetype": "true"})
+    assert h.isdir(server.address + "/index/")
+    assert not h.isdir(server.realfile)
+    assert not h.isdir(server.address + "doesnotevenexist")
+
     h = fsspec.filesystem("http")
     assert h.isdir(server.address + "/index/")
     assert not h.isdir(server.realfile)
@@ -443,7 +448,11 @@ def test_docstring():
 def test_async_other_thread(server):
     import threading
 
-    loop = asyncio.get_event_loop()
+    try:
+        loop = asyncio.get_event_loop()
+    except RuntimeError:  # Python 3.14+ codepath
+        loop = asyncio.new_event_loop()
+
     th = threading.Thread(target=loop.run_forever)
 
     th.daemon = True
