@@ -15,6 +15,7 @@ from fsspec.implementations.ftp import FTPFileSystem
 from fsspec.implementations.http import HTTPFileSystem
 from fsspec.implementations.local import LocalFileSystem
 from fsspec.spec import AbstractBufferedFile, AbstractFileSystem
+from fsspec.tests.conftest import data
 
 PATHS_FOR_GLOB_TESTS = (
     {"name": "test0.json", "type": "file", "size": 100},
@@ -744,6 +745,31 @@ def test_cache():
     assert len(DummyTestFS._cache) == 0
 
 
+def test_cache_not_pickled(server):
+    fs = fsspec.filesystem(
+        "http",
+        cache_type="readahead",
+        headers={"give_length": "true", "head_ok": "true"},
+    )
+    filepath = server.realfile
+    length = 3
+    f = fs.open(filepath, mode="rb")
+    assert isinstance(f, AbstractBufferedFile)
+    assert not f.cache.cache  # No cache initially
+    assert f.read(length=length) == data[:length]
+    assert f.cache.cache == data  # Cache is populated
+
+    # Roundtrip through pickle
+    import pickle
+
+    f2 = pickle.loads(pickle.dumps(f))
+    assert not f2.cache.cache  # No cache initially
+    assert (
+        f2.read(length=length) == data[length : 2 * length]
+    )  # Read file from previous seek point
+    assert f2.cache.cache == data[length:]  # Cache is populated
+
+
 def test_current():
     fs = DummyTestFS()
     fs2 = DummyTestFS(arg=1)
